@@ -38,8 +38,11 @@
 #include <hildon/hildon-file-system-model.h>
 #include <libosso.h>      /* required for screen saver timeout */
 #define GTK_FM_OK  GTK_RESPONSE_OK
+#define HILDON_ENTRY_NO_AUTOCAP(a) \
+  hildon_gtk_entry_set_input_mode(GTK_ENTRY(a),HILDON_GTK_INPUT_MODE_FULL)
 #else
 #define GTK_FM_OK  GTK_RESPONSE_ACCEPT
+#define HILDON_ENTRY_NO_AUTOCAP(a)
 #endif
 
 
