@@ -165,3 +165,12 @@ canvas_item_t *canvas_image_new(map_t *map, canvas_group_t group,
   goo_canvas_item_scale(item, hscale, vscale);
   return item;
 }
+
+void canvas_image_move(canvas_item_t *item, gint x, gint y, 
+		       float hscale, float vscale) {
+
+  g_object_set(G_OBJECT(item), 
+	       "x", (gdouble)x / hscale, 
+	       "y", (gdouble)y / vscale, 
+	       NULL);  
+}
