@@ -27,6 +27,10 @@ static void on_node_add_clicked(GtkButton *button, gpointer data) {
   map_action_set((appdata_t*)data, MAP_ACTION_NODE_ADD);
 }
 
+static void on_way_add_clicked(GtkButton *button, gpointer data) {
+  map_action_set((appdata_t*)data, MAP_ACTION_WAY_ADD);
+}
+
 static void on_trash_clicked(GtkButton *button, gpointer data) {
   map_delete_selected((appdata_t*)data);
 }
@@ -66,6 +70,7 @@ void icon_bar_map_action_idle(appdata_t *appdata, gboolean idle) {
   gint i;
   GtkWidget *action_widgets[] = {
     appdata->iconbar->node_add,
+    appdata->iconbar->way_add,
     NULL
   };
 
@@ -102,6 +107,8 @@ GtkWidget *iconbar_new(appdata_t *appdata) {
 			    on_trash_clicked);
   iconbar->node_add = icon_add(vbox, appdata, "node_add",
 			       on_node_add_clicked);
+  //  iconbar->way_add = icon_add(vbox, appdata, "way_add",
+  //			      on_way_add_clicked);
 
   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
   iconbar->ok = icon_add(hbox, appdata, "ok", on_ok_clicked);
