@@ -20,6 +20,16 @@
 #ifndef INFO_H
 #define INFO_H
 
+typedef struct {
+  appdata_t *appdata;
+  GtkWidget *dialog, *view;
+  GtkListStore *store;
+  GtkWidget *but_add, *but_remove, *but_edit;
+  tag_t **tag;
+} tag_context_t;
+
+
 void info_dialog(appdata_t *appdata);
+void info_tags_replace(tag_context_t *context, tag_t *tag);
 
 #endif // INFO_H
