@@ -102,6 +102,14 @@ static presets_item_t *parse_item(xmlDocPtr doc, xmlNode *a_node) {
 	*widget = g_new0(presets_widget_t, 1);
 	(*widget)->type = WIDGET_TYPE_LABEL;
 	(*widget)->text = (char*)xmlGetProp(cur_node, BAD_CAST "text");
+
+	/* special handling of separators */
+	if(!(*widget)->text || (strcmp((*widget)->text, " ") == 0)) {
+	  (*widget)->type = WIDGET_TYPE_SEPARATOR;
+	  if((*widget)->text) xmlFree((*widget)->text);
+	  (*widget)->text = NULL;
+	}
+
 	widget = &((*widget)->next);
 
       } else if(strcasecmp((char*)cur_node->name, "text") == 0) {
@@ -264,15 +272,18 @@ presets_item_t *josm_presets_load(void) {
 /* --------------------- the items dialog -------------------- */
 
 static void attach_both(GtkWidget *table, GtkWidget *widget, gint y) {
-  gtk_table_attach_defaults(GTK_TABLE(table), widget, 0,2,y,y+1);
+  gtk_table_attach(GTK_TABLE(table), widget, 0,2,y,y+1, 
+		   GTK_EXPAND | GTK_FILL, 0,0,0);
 }
 
 static void attach_text(GtkWidget *table, char *text, gint y) {
-  gtk_table_attach_defaults(GTK_TABLE(table), gtk_label_new(text), 0,1,y,y+1);
+  gtk_table_attach(GTK_TABLE(table), gtk_label_new(text), 0,1,y,y+1, 
+		   GTK_EXPAND | GTK_FILL, 0,0,0);
 }
 
 static void attach_right(GtkWidget *table, GtkWidget *widget, gint y) {
-  gtk_table_attach_defaults(GTK_TABLE(table), widget, 1,2,y,y+1);
+  gtk_table_attach(GTK_TABLE(table), widget, 1,2,y,y+1, 
+		   GTK_EXPAND | GTK_FILL, 0,0,0);
 }
 
 static tag_t **store_value(presets_widget_t *widget, tag_t **ctag, char *value) {
@@ -291,7 +302,23 @@ static tag_t **store_value(presets_widget_t *widget, tag_t **ctag, char *value)
   return ctag;
 }
 
-static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
+#ifdef USE_HILDON
+static gint table_expose_event(GtkWidget *widget, GdkEventExpose *event, 
+			 gboolean *first) {
+
+  if(*first) {
+    guint border_width = gtk_container_get_border_width(GTK_CONTAINER(widget->parent));
+    gtk_viewport_set_shadow_type(GTK_VIEWPORT(widget->parent), GTK_SHADOW_NONE);
+
+    gtk_widget_set_size_request(GTK_WIDGET(widget->parent), -1, 
+				widget->allocation.height +  2*border_width);
+    *first = FALSE;
+  }
+  return FALSE;
+}
+#endif
+
+static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item, tag_t *orig_tag) {
   gboolean ok = FALSE;
   tag_t *tag = NULL, **ctag = &tag;
 
@@ -307,11 +334,12 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
   /* count total number of widgets and number of widgets that */
   /* have an interactive gui element. We won't show a dialog */
   /* at all if there's no interactive gui element at all */
-  gint widget_cnt = 0, gui_widget_cnt = 0;
+  gint widget_cnt = 0, interactive_widget_cnt = 0;
   while(widget) {
     if((widget->type != WIDGET_TYPE_LABEL) &&
+       (widget->type != WIDGET_TYPE_SEPARATOR) &&
        (widget->type != WIDGET_TYPE_KEY))
-      gui_widget_cnt++;
+      interactive_widget_cnt++;
 
     widget_cnt++;
     widget = widget->next;
@@ -320,30 +348,49 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
   /* allocate space for required number of gtk widgets */
   GtkWidget **gtk_widgets = (GtkWidget**)g_new0(GtkWidget, widget_cnt);
 
-  if(gui_widget_cnt)  {
-    /* create table of required size */
-    GtkWidget *table = gtk_table_new(widget_cnt, 2, FALSE);
-  
+  if(interactive_widget_cnt)  {
+    /* special handling for the first label/separators */
+    guint widget_skip = 0;  // number of initial widgets to skip
     widget = item->widget;
-    widget_cnt = 0;
+    if(widget && (widget->type == WIDGET_TYPE_LABEL)) {
+      gtk_window_set_title(GTK_WINDOW(dialog), widget->text);
+
+      widget_skip++;   // this widget isn't part of the contents anymore
+      widget = widget->next;
+
+      /* skip all following separators */
+      while(widget && (widget->type == WIDGET_TYPE_SEPARATOR)) {
+	widget_skip++;   // this widget isn't part of the contents anymore
+	widget = widget->next;
+      }
+    }
+
+    /* create table of required size */
+    GtkWidget *table = gtk_table_new(widget_cnt-widget_skip, 2, FALSE);
+  
+    widget_cnt = widget_skip;
     while(widget) {
+      /* check if there's a value with this key already */
+      char *preset = osm_tag_get_by_key(orig_tag, widget->key);
+
       switch(widget->type) {
+      case WIDGET_TYPE_SEPARATOR:
+	attach_both(table, gtk_hseparator_new(), widget_cnt-widget_skip);
+	break;
+
       case WIDGET_TYPE_LABEL:
-	if(strcmp(widget->text, " ") == 0)
-	  attach_both(table, gtk_hseparator_new(), widget_cnt);
-	else 
-	  attach_both(table, gtk_label_new(widget->text), widget_cnt);
+	attach_both(table, gtk_label_new(widget->text), widget_cnt-widget_skip);
 	break;
 	
       case WIDGET_TYPE_COMBO:
-	attach_text(table, widget->text, widget_cnt);
-	
+	attach_text(table, widget->text, widget_cnt-widget_skip);
+
+	if(!preset && widget->combo_w.def) preset = widget->combo_w.def;
 	gtk_widgets[widget_cnt] = gtk_combo_box_new_text();
 	presets_value_t *value = widget->combo_w.values;
 	int count = 0, active = -1;
 	while(value) {
-	  if(active < 0 && widget->combo_w.def && 
-	     strcmp(widget->combo_w.def, value->text)==0)
+	  if(active < 0 && preset && strcmp(preset, value->text)==0)
 	    active = count;
 	  
 	  gtk_combo_box_append_text(GTK_COMBO_BOX(gtk_widgets[widget_cnt]), 
@@ -354,25 +401,28 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
 	
 	gtk_combo_box_set_active(GTK_COMBO_BOX(gtk_widgets[widget_cnt]), 
 				 active);
-	attach_right(table, gtk_widgets[widget_cnt], widget_cnt);
+	attach_right(table, gtk_widgets[widget_cnt], widget_cnt-widget_skip);
 	break;
 	
       case WIDGET_TYPE_CHECK:
+	{ gboolean def = FALSE;
+	if(preset) def = strcasecmp(preset, "true");
+	else       def = widget->check_w.def;
+
 	gtk_widgets[widget_cnt] = gtk_check_button_new_with_label(widget->text);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_widgets[widget_cnt]), 
-				     widget->check_w.def);
-	attach_right(table, gtk_widgets[widget_cnt], widget_cnt);
-      break;
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtk_widgets[widget_cnt]), def);
+	attach_right(table, gtk_widgets[widget_cnt], widget_cnt-widget_skip);
+      } break;
       
     case WIDGET_TYPE_TEXT:
-      attach_text(table, widget->text, widget_cnt);
+      attach_text(table, widget->text, widget_cnt-widget_skip);
       
+      if(!preset && widget->text_w.def) preset = widget->text_w.def;
       gtk_widgets[widget_cnt] = gtk_entry_new();
-      if(widget->text_w.def)
-	gtk_entry_set_text(GTK_ENTRY(gtk_widgets[widget_cnt]), 
-			   widget->text_w.def);
+      if(preset)
+	gtk_entry_set_text(GTK_ENTRY(gtk_widgets[widget_cnt]), preset);
 
-      attach_right(table, gtk_widgets[widget_cnt], widget_cnt);
+      attach_right(table, gtk_widgets[widget_cnt], widget_cnt-widget_skip);
       break;
       
       default:
@@ -386,25 +436,27 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
 #ifndef USE_HILDON
     /* add widget to dialog */
     gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
+    gtk_window_set_default_size(GTK_WINDOW(dialog), 300, 50);
 #else
     /* put it into a scrolled window */
     GtkWidget *scroll_win = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_win), 
 				   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll_win), 
-					  table);
+    					  table);
 
-    gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
-				scroll_win);
-    
-    gtk_window_set_default_size(GTK_WINDOW(dialog), 50, 400);
+    gboolean first = TRUE;
+    gtk_signal_connect(GTK_OBJECT(table), "expose_event",
+		       G_CALLBACK(table_expose_event), &first);
+
+    gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), scroll_win);
+    //    gtk_window_set_default_size(GTK_WINDOW(dialog), 50, 400);
 #endif
     
-    
     gtk_widget_show_all(dialog);
   }
     
-  if(!gui_widget_cnt || 
+  if(!interactive_widget_cnt || 
      (GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(dialog)))) {
     ok = TRUE;
 
@@ -460,7 +512,7 @@ static tag_t *presets_item_dialog(GtkWindow *parent, presets_item_t *item) {
 
   g_free(gtk_widgets);
 
-  if(gui_widget_cnt)
+  if(interactive_widget_cnt)
     gtk_widget_destroy(dialog);
 
   return tag;
@@ -482,7 +534,8 @@ cb_menu_item(GtkMenuItem *menu_item, gpointer data) {
   g_assert(item);
 
   tag_t *tag = 
-    presets_item_dialog(GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(menu_item))), item);
+    presets_item_dialog(GTK_WINDOW(context->tag_context->dialog), item,
+			*context->tag_context->tag);
 
   if(tag) {
     tag_context_t *tag_context = context->tag_context;
