@@ -154,11 +154,26 @@ cb_menu_fullscreen(GtkWidget *widget, gpointer data) {
   else
     gtk_window_unfullscreen(GTK_WINDOW(appdata->window));
 }
+
+static void 
+cb_menu_zoomin(GtkWidget *widget, appdata_t *appdata) {
+  if(!appdata || !appdata->map) return;
+
+  map_set_zoom(appdata->map, appdata->map->zoom*2);
+  printf("zoom is now %f\n", appdata->map->zoom);
+}
+
+static void 
+cb_menu_zoomout(GtkWidget *widget, appdata_t *appdata) {
+  if(!appdata || !appdata->map) return;
+
+  map_set_zoom(appdata->map, appdata->map->zoom/2);
+  printf("zoom is now %f\n", appdata->map->zoom);
+}
 #endif
 
 static void 
-cb_menu_track_import(GtkWidget *window, gpointer data) {
-  appdata_t *appdata = (appdata_t*)data;
+cb_menu_track_import(GtkWidget *window, appdata_t *appdata) {
 
   /* open a file selector */
   GtkWidget *dialog;
@@ -246,15 +261,30 @@ void menu_create(appdata_t *appdata) {
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_fullscreen), 
 		   appdata);
+
+  item = gtk_menu_item_new_with_label( _("Zoom +" ));
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_zoomin), appdata);
+
+  item = gtk_menu_item_new_with_label( _("Zoom -") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_zoomout), appdata);
 #endif
 
-  item = gtk_menu_item_new_with_label( _("Upload...") );
+  /* -------------------- OSM submenu -------------------- */
+
+  item = gtk_menu_item_new_with_label( _("OSM") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
+  submenu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
+
+  item = gtk_menu_item_new_with_label( _("Upload...") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_upload), appdata);
 
 #if 1 // testing
   item = gtk_menu_item_new_with_label( _("Redraw...") );
-  gtk_menu_append(GTK_MENU_SHELL(menu), item);
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_redraw), appdata);
 #endif
 
@@ -546,10 +576,13 @@ int main(int argc, char *argv[]) {
   wms_load(&appdata);
 
   /* give main window a new name */
+#ifndef USE_HILDON
   char *str = g_strdup_printf("OSM2Go - %s", appdata.project->name);
   gtk_window_set_title(GTK_WINDOW(appdata.window), str);
   g_free(str);
-
+#else
+  gtk_window_set_title(GTK_WINDOW(appdata.window), appdata.project->name);
+#endif
 
   /* ------------ jump into main loop ---------------- */
 
