@@ -111,6 +111,11 @@ static void map_statusbar(map_t *map, map_item_t *map_item) {
   g_free(str);
 }
 
+static void map_outside_error(appdata_t *appdata) {
+  errorf(GTK_WIDGET(appdata->window), 
+	 _("Items must not be placed outside the working area!"));
+}
+
 static void map_node_select(appdata_t *appdata, node_t *node) {
   map_t *map = appdata->map;
   map_item_t *map_item = &map->selected;
@@ -545,7 +550,7 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
   return map_item;
 }
 
-static void map_set_zoom(map_t *map, double zoom) {
+void map_set_zoom(map_t *map, double zoom) {
   map->zoom = zoom;
   canvas_set_zoom(map->canvas, map->zoom);
 }
@@ -662,6 +667,17 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 	 node->lpos.x, node->lpos.y,
 	 node->pos.lat, node->pos.lon);
 	    
+  /* convert mouse position to canvas (world) position */
+  gint x, y;
+  canvas_window2world(map->canvas, ex, ey, &x, &y);
+  if(!osm_position_within_bounds(appdata->osm, x, y)) {
+    map_outside_error(appdata);
+    return;
+  }
+
+  node->lpos.x = x;
+  node->lpos.y = y;
+
   /* convert mouse position to canvas (world) position */
   canvas_window2world(map->canvas, ex, ey, &node->lpos.x, &node->lpos.y);
 	    
@@ -753,6 +769,37 @@ static void map_handle_click(appdata_t *appdata, map_t *map) {
   }
 }
 
+static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
+  map_t *map = appdata->map;
+
+  map_hl_touchnode_clear(map);
+
+#if 0
+  /* check if we are close to another node */
+  g_assert(map->pen_down.on_item);
+  g_assert(map->pen_down.on_item->type == MAP_TYPE_NODE);
+#endif
+
+  /* check if we are close to one of the other nodes */
+  canvas_window2world(appdata->map->canvas, x, y, &x, &y);
+  node_t *node = appdata->osm->node;
+  while(node) {
+    /* don't highlight the dragged node itself and don't highlight */
+    /* deleted ones */
+    if(((!map->pen_down.on_item ||
+	 (node != map->pen_down.on_item->node))) && 
+       (!(node->flags & OSM_FLAG_DELETED))) {
+      gint nx = x - node->lpos.x;
+      gint ny = y - node->lpos.y;
+	  
+      if(nx*nx + ny*ny < RADIUS*RADIUS)
+	map_hl_touchnode_draw(map, node->lpos.x, node->lpos.y);
+
+    }
+    node = node->next;
+  }
+}
+
 static void map_button_press(map_t *map, gint x, gint y) {
 
   printf("left button pressed\n");
@@ -779,10 +826,18 @@ static void map_button_press(map_t *map, gint x, gint y) {
     map->pen_down.on_selected_node = 
       item_is_selected_node(map, map->pen_down.on_item);
 
-  switch(map->action) {
+  /* button press */
+  switch(map->action.type) {
+
   case MAP_ACTION_NODE_ADD:
     map_hl_cursor_draw(map, x, y);
     break;
+
+  case MAP_ACTION_WAY_ADD:
+    map_hl_cursor_draw(map, x, y);
+    map_touchnode_update(map->appdata, x, y);
+    break;
+
   default:
     break;
   }
@@ -805,7 +860,8 @@ static void map_bg_adjust(map_t *map, gint x, gint y) {
 static void map_button_release(map_t *map, gint x, gint y) {
   map->pen_down.is = FALSE;
 
-  switch(map->action) {
+  /* before button release is handled */
+  switch(map->action.type) {
   case MAP_ACTION_BG_ADJUST:
     map_bg_adjust(map, x, y);
     map->bg_offset.x += x - map->pen_down.at.x;
@@ -864,13 +920,53 @@ static void map_button_release(map_t *map, gint x, gint y) {
 
     /* convert mouse position to canvas (world) position */
     canvas_window2world(map->canvas, x, y, &x, &y);
-    node_t *node = osm_node_new(map->appdata->osm, x, y);
-    map_node_draw(map, node);
 
+    node_t *node = NULL;
+    if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
+      map_outside_error(map->appdata);
+    else {
+      node = osm_node_new(map->appdata->osm, x, y);
+      map_node_draw(map, node);
+    }
     map_action_set(map->appdata, MAP_ACTION_IDLE);
-
+      
     map_item_deselect(map->appdata);
-    map_node_select(map->appdata, node);
+
+    if(node)
+      map_node_select(map->appdata, node);
+    break;
+
+  case MAP_ACTION_WAY_ADD:
+    printf("released after WAY ADD\n");
+    map_hl_cursor_clear(map);
+
+    /* convert mouse position to canvas (world) position */
+    canvas_window2world(map->canvas, x, y, &x, &y);
+
+    node = NULL;
+    if(!osm_position_within_bounds(map->appdata->osm, x, y)) 
+      map_outside_error(map->appdata);
+    else {
+      g_assert(map->action.way);
+
+      node = osm_node_new(map->appdata->osm, x, y);
+      osm_way_append_node(map->action.way, node);
+
+      switch(osm_node_chain_length(map->action.way->node_chain)) {
+      case 1:
+	/* replace "place first node..." message */
+	statusbar_set(map->appdata, _("Place next node of way"));
+	break;
+	
+      case 2:
+	/* two nodes are enough for a valid way */
+	icon_bar_map_cancel_ok(map->appdata, TRUE, TRUE);
+	break;
+	
+      default:
+	break;
+      }
+    }
     break;
 
   default:
@@ -896,35 +992,6 @@ static gboolean map_button_event(GtkWidget *widget, GdkEventButton *event,
   return FALSE;  /* forward to further processing */
 }
 
-static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
-  map_t *map = appdata->map;
-
-  map_hl_touchnode_clear(map);
-
-  /* check if we are close to another node */
-  g_assert(map->pen_down.on_item);
-  g_assert(map->pen_down.on_item->type == MAP_TYPE_NODE);
-
-  /* check if we are close to one of the other nodes */
-  canvas_window2world(appdata->map->canvas, x, y, &x, &y);
-  node_t *node = appdata->osm->node;
-  while(node) {
-    /* don't highlight the dragged node itself and don't highlight */
-    /* deleted ones */
-    if((node != map->pen_down.on_item->node) && 
-       (!(node->flags & OSM_FLAG_DELETED))) {
-      gint nx = x - node->lpos.x;
-      gint ny = y - node->lpos.y;
-	  
-      if(nx*nx + ny*ny < RADIUS*RADIUS)
-	map_hl_touchnode_draw(map, node->lpos.x, node->lpos.y);
-
-    }
-    node = node->next;
-  }
-}
-
-
 static gboolean map_motion_notify_event(GtkWidget *widget, 
                              GdkEventMotion *event, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
@@ -959,7 +1026,8 @@ static gboolean map_motion_notify_event(GtkWidget *widget,
   if(!map->pen_down.drag)
     map->pen_down.drag = distance_above(map, x, y, MAP_DRAG_LIMIT);
   
-  switch(map->action) {
+  /* drag */
+  switch(map->action.type) {
   case MAP_ACTION_BG_ADJUST:
     map_bg_adjust(map, x, y);
     break;
@@ -980,6 +1048,11 @@ static gboolean map_motion_notify_event(GtkWidget *widget,
     map_hl_cursor_draw(map, x, y);
     break;
     
+  case MAP_ACTION_WAY_ADD:
+    map_hl_cursor_draw(map, x, y);
+    map_touchnode_update(appdata, x, y);
+    break;
+
   default:
     break;
   }
@@ -1030,7 +1103,7 @@ GtkWidget *map_new(appdata_t *appdata) {
   map->pen_down.at.x = -1;
   map->pen_down.at.y = -1;
   map->appdata = appdata;
-  map->action = MAP_ACTION_IDLE;
+  map->action.type = MAP_ACTION_IDLE;
 
 #ifdef USE_GNOMECANVAS
   map->canvas = gnome_canvas_new_aa();  // _aa
@@ -1155,21 +1228,14 @@ void map_paint(appdata_t *appdata) {
 void map_action_set(appdata_t *appdata, map_action_t action) {
   printf("map action set to %d\n", action);
 
-  /* returning from this action */
-  switch(appdata->map->action) {
-  case MAP_ACTION_BG_ADJUST:
-    gtk_widget_set_sensitive(appdata->menu_item_wms_adjust, TRUE);
-    break;
-  default:
-    break;
-  }
-
-  appdata->map->action = action;
+  appdata->map->action.type = action;
 
   /* enable/disable ok/cancel buttons */
-  // MAP_ACTION_IDLE=0, _NODE_ADD, _BG_ADJUST
-  const gboolean ok_state[] = { FALSE, FALSE, TRUE };
-  const gboolean cancel_state[] = { FALSE, TRUE, TRUE };
+  // MAP_ACTION_IDLE=0, _NODE_ADD, _BG_ADJUST, _WAY_ADD
+  const gboolean ok_state[] = { FALSE, FALSE, TRUE, FALSE };
+  const gboolean cancel_state[] = { FALSE, TRUE, TRUE, TRUE };
+
+  g_assert(MAP_ACTION_NUM == sizeof(ok_state)/sizeof(gboolean));
   g_assert(action < sizeof(ok_state)/sizeof(gboolean));
 
   icon_bar_map_cancel_ok(appdata, cancel_state[action], ok_state[action]);
@@ -1177,36 +1243,51 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
   switch(action) {
   case MAP_ACTION_BG_ADJUST:
     /* an existing selection only causes confusion ... */
-    gtk_widget_set_sensitive(appdata->menu_item_wms_adjust, FALSE);
     map_item_deselect(appdata);
     break;
 
+  case MAP_ACTION_WAY_ADD:
+    printf("starting new way\n");
+    g_assert(!appdata->map->action.way);
+    appdata->map->action.way = g_new0(way_t, 1);
+    break;
+
   default:
     break;
   }
 
   icon_bar_map_action_idle(appdata, action == MAP_ACTION_IDLE);
+  gtk_widget_set_sensitive(appdata->menu_item_wms_adjust, 
+			   action == MAP_ACTION_IDLE);
 
   const char *str_state[] = { 
     NULL, 
-    _("Please place a node"),
+    _("Place a node"),
     _("Adjust background image position"),
+    _("Place first node of new way"),
   };
 
+  g_assert(MAP_ACTION_NUM == sizeof(str_state)/sizeof(char*));
+
   statusbar_set(appdata, str_state[action]);
 }
 
 void map_action_cancel(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  switch(map->action) {
+  switch(map->action.type) {
+  case MAP_ACTION_WAY_ADD:
+    osm_way_free(map->action.way);
+    map->action.way = NULL;
+    break;
+
   case MAP_ACTION_BG_ADJUST:
     /* undo all changes to bg_offset */
     map->bg_offset.x = appdata->project->wms_offset.x;
     map->bg_offset.y = appdata->project->wms_offset.y;
 
-    gint x = map->appdata->osm->bounds->min.x + map->bg_offset.x;
-    gint y = map->appdata->osm->bounds->min.y + map->bg_offset.y;
+    gint x = appdata->osm->bounds->min.x + map->bg_offset.x;
+    gint y = appdata->osm->bounds->min.y + map->bg_offset.y;
     canvas_image_move(map->bg_item, x, y, map->bg_scale.x, map->bg_scale.y);
     break;
 
@@ -1220,7 +1301,13 @@ void map_action_cancel(appdata_t *appdata) {
 void map_action_ok(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  switch(map->action) {
+  switch(map->action.type) {
+  case MAP_ACTION_WAY_ADD:
+    printf("TODO: convert temporary way into real one\n");
+    osm_way_free(map->action.way);
+    map->action.way = NULL;
+    break;
+
   case MAP_ACTION_BG_ADJUST:
     /* save changes to bg_offset in project */
     appdata->project->wms_offset.x = map->bg_offset.x;
