@@ -55,10 +55,14 @@ typedef enum {
   MAP_TYPE_WAY 
 } map_type_t;
 
+/* don't reorder these as some things in map.c */
+/* rely on a certain order */
 typedef enum { 
   MAP_ACTION_IDLE=0, 
   MAP_ACTION_NODE_ADD, 
-  MAP_ACTION_BG_ADJUST 
+  MAP_ACTION_BG_ADJUST,
+  MAP_ACTION_WAY_ADD,
+  MAP_ACTION_NUM
 } map_action_t;
 
 typedef struct map_highlight_s {
@@ -105,7 +109,13 @@ typedef struct map_s {
   struct { float x, y; } bg_offset;
   struct { float x, y; } bg_scale;
 
-  map_action_t action;             // current action in progress
+  struct {
+    map_action_t type;            // current action type in progress
+
+    union {                       // action related variables/states
+      way_t *way;
+    };
+  } action;
 
   /* variables required for pen/mouse handling */
   struct {
@@ -151,5 +161,6 @@ void map_track_pos(appdata_t *appdata, lpos_t *lpos);
 /* background stuff */
 void map_set_bg_image(map_t *map, char *filename);
 void map_remove_bg_image(map_t *map);
+void map_set_zoom(map_t *map, double zoom);
 
 #endif // MAP_H
