@@ -26,12 +26,15 @@ void map_hl_cursor_clear(map_t *map);
 void map_hl_touchnode_draw(map_t *map, gint x, gint y);
 void map_hl_touchnode_clear(map_t *map);
 
-GnomeCanvasItem *map_hl_new(map_t *map, 
-				   GnomeCanvasGroup *parent, GType type, 
-				   map_item_t *map_item,
-				   const gchar *first_arg_name,
-				   ...);
 void map_hl_remove(appdata_t *appdata);
 gboolean map_hl_item_is_highlighted(map_t *map, map_item_t *item);
 
+canvas_item_t *map_hl_circle_new(map_t *map, canvas_group_t group, map_item_t *map_item, 
+				 gint x, gint y, gint radius, canvas_color_t color);
+
+canvas_item_t *map_hl_polyline_new(map_t *map, canvas_group_t group, map_item_t *map_item, 
+				   canvas_points_t *points, gint width, canvas_color_t color);
+
+
+
 #endif // MAP_HL_H
