@@ -339,22 +339,46 @@ void osm_node_chain_free(node_chain_t *node_chain) {
   }
 }
 
+void osm_way_free(way_t *way) {
+  osm_node_chain_free(way->node_chain);
+  osm_tags_free(way->tag);
+
+  /* there must not be anything left in this chain */
+  g_assert(!way->map_item_chain);
+
+  g_free(way);
+}
+
 static void osm_ways_free(way_t *way) {
   while(way) {
     way_t *next = way->next;
-
-    osm_node_chain_free(way->node_chain);
-    osm_tags_free(way->tag);
-
-    /* there must not be anything left in this chain */
-    g_assert(!way->map_item_chain);
-
-    g_free(way);
-
+    osm_way_free(way);
     way = next;
   }
 }
 
+void osm_way_append_node(way_t *way, node_t *node) {
+  node_chain_t **node_chain = &way->node_chain;
+
+  while(*node_chain) 
+    node_chain = &((*node_chain)->next);
+
+  *node_chain = g_new0(node_chain_t, 1);
+  (*node_chain)->node = node;
+
+  node->ways++;
+}
+
+int osm_node_chain_length(node_chain_t *node_chain) {
+  int cnt = 0;
+  while(node_chain) {
+    cnt++;
+    node_chain = node_chain->next;
+  }
+
+  return cnt;
+}
+
 void osm_way_dump(way_t *way) {
   char buf[64];
   struct tm tm;
@@ -826,6 +850,19 @@ gboolean osm_sanity_check(GtkWidget *parent, osm_t *osm) {
 
 /* ------------------------- misc access functions -------------- */
 
+char *osm_tag_get_by_key(tag_t *tag, char *key) {
+  if(!tag || !key) return NULL;
+
+  while(tag) {
+    if(strcasecmp(tag->key, key) == 0)
+      return tag->value;
+
+    tag = tag->next;
+  }
+
+  return NULL;
+}
+
 char *osm_way_get_value(way_t *way, char *key) {
   tag_t *tag = way->tag;
 
@@ -1231,3 +1268,9 @@ way_chain_t *osm_node_to_way(osm_t *osm, node_t *node) {
 
   return chain;
 }
+
+gboolean osm_position_within_bounds(osm_t *osm, gint x, gint y) {
+  if((x < osm->bounds->min.x) || (x > osm->bounds->max.x)) return FALSE;
+  if((y < osm->bounds->min.y) || (y > osm->bounds->max.y)) return FALSE;
+  return TRUE;
+}
