@@ -149,20 +149,27 @@ node_chain_t *osm_parse_osm_way_nd(osm_t *osm, xmlDocPtr doc, xmlNode *a_node);
 void osm_dump(osm_t *osm);
 void osm_free(osm_t *osm);
 
-gboolean osm_way_has_key_or_value(way_t *way, char *str);
-gboolean osm_way_is_way(way_t *way);
-char *osm_way_get_value(way_t *way, char *key);
 char *osm_node_get_value(node_t *node, char *key);
 gboolean osm_node_has_tag(node_t *node);
 
 void osm_node_dump(node_t *node);
+
+void osm_way_free(way_t *way);
 void osm_way_dump(way_t *way);
+gboolean osm_way_is_way(way_t *way);
+char *osm_way_get_value(way_t *way, char *key);
+gboolean osm_way_has_key_or_value(way_t *way, char *str);
+void osm_way_append_node(way_t *way, node_t *node);
+
+gboolean osm_node_in_way(way_t *way, node_t *node);
+
+void osm_node_chain_free(node_chain_t *node_chain);
+int osm_node_chain_length(node_chain_t *node_chain);
+
 
 void osm_tag_free(tag_t *tag);
 void osm_tags_free(tag_t *tag);
-void osm_node_chain_free(node_chain_t *node_chain);
-
-gboolean osm_node_in_way(way_t *way, node_t *node);
+char *osm_tag_get_by_key(tag_t *tag, char *key);
 
 char *osm_generate_xml_node(osm_t *osm, node_t *node);
 char *osm_generate_xml_way(osm_t *osm, way_t *way);
@@ -177,5 +184,6 @@ way_chain_t *osm_node_to_way(osm_t *osm, node_t *node);
 /* ----------- edit functions ----------- */
 node_t *osm_node_new(osm_t *osm, gint x, gint y);
 way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently);
+gboolean osm_position_within_bounds(osm_t *osm, gint x, gint y);
 
 #endif /* OSM_H */
