@@ -638,10 +638,12 @@ void osm_upload(appdata_t *appdata, osm_t *osm, project_t *project) {
   table = gtk_table_new(2, 2, FALSE);
   table_attach_label_l(table, _("Username:"), 0, 1, 0, 1);
   GtkWidget *uentry = gtk_entry_new();
+  HILDON_ENTRY_NO_AUTOCAP(uentry);
   gtk_entry_set_text(GTK_ENTRY(uentry), project->username);
   gtk_table_attach_defaults(GTK_TABLE(table),  uentry, 1, 2, 0, 1);
   table_attach_label_l(table, _("Password:"), 0, 1, 1, 2);
   GtkWidget *pentry = gtk_entry_new();
+  HILDON_ENTRY_NO_AUTOCAP(pentry);
   gtk_entry_set_text(GTK_ENTRY(pentry), project->password);
   gtk_entry_set_visibility(GTK_ENTRY(pentry), FALSE);
   gtk_table_attach_defaults(GTK_TABLE(table),  pentry, 1, 2, 1, 2);
