@@ -328,7 +328,7 @@ view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
     
   if(gtk_tree_model_get_iter(model, &iter, path)) {
     g_assert(gtk_tree_path_get_depth(path) == 1);
-    view_selected(context, !path_currently_selected);
+    view_selected(context, TRUE);
   }
   
   return TRUE; /* allow selection state to change */
@@ -941,6 +941,7 @@ gboolean project_edit(GtkWidget *parent, project_t *project POS_PARM) {
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
   gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 4, 5);
   context->server = gtk_entry_new();
+  HILDON_ENTRY_NO_AUTOCAP(context->server);
   gtk_entry_set_text(GTK_ENTRY(context->server), project->server);
   gtk_table_attach_defaults(GTK_TABLE(table),  context->server, 1, 4, 4, 5);
 
