@@ -37,11 +37,24 @@ typedef struct project_s {
 
   pos_t min, max;
 
-  gboolean data_dirty;   /* needs to download new data */
-  gboolean dirty;        /* project file needs to be written */
+  gboolean data_dirty;     /* needs to download new data */
+  gboolean dirty;          /* project file needs to be written */
+
+  struct project_s *next;  /* for chaining projects (e.g. during scan) */
 } project_t;
 
-project_t *project_open(appdata_t *appdata, char *name);
-void project_save(appdata_t *appdata, project_t *project);
+char *project_select(appdata_t *appdata);
+gboolean project_open(appdata_t *appdata, char *name);
+gboolean project_save(GtkWidget *parent, project_t *project);
+
+#ifdef USE_HILDON
+#define POS_PARM  ,dbus_mm_pos_t *mmpos, osso_context_t *osso_context
+#else
+#define POS_PARM
+#endif
+
+gboolean project_edit(GtkWidget *parent, project_t *project POS_PARM);
+
+void project_free(project_t *project);
 
 #endif // PROJECT_H
